package dev.fitko.fitconnect.core.portal;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.config.Environment;
import dev.fitko.fitconnect.api.config.EnvironmentName;
import dev.fitko.fitconnect.api.domain.model.jwk.ApiJwk;
import dev.fitko.fitconnect.api.domain.model.jwk.ApiJwkSet;
import dev.fitko.fitconnect.api.services.portal.SelfServicePortalService;
import dev.fitko.fitconnect.core.RestEndpointBase;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.http.MediaType;
import org.springframework.web.client.RestTemplate;

import java.util.List;
import java.util.Map;

import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;

class SelfServicePortalApiServiceTest extends RestEndpointBase {

    private ApplicationConfig config;
    private SelfServicePortalService underTest;

    @BeforeEach
    void setup() {

        final var fakeBaseUrl = "http://localhost:" + wireMockServer.port();

        final EnvironmentName envName = new EnvironmentName("TESTING");
        final Environment environment = new Environment();
        environment.setSubmissionBaseUrl(fakeBaseUrl);

        this.config = new ApplicationConfig();
        this.config.setEnvironments(Map.of(envName, environment));
        this.config.setActiveEnvironment(envName);

        this.underTest = new SelfServicePortalApiService(this.config, new RestTemplate());
    }

    @Test
    void testGetPortalPublicKeys() throws JsonProcessingException {

        // Given
        final ApiJwk portalSignatureKey = new ApiJwk();

        final ApiJwkSet apiJwkSet = new ApiJwkSet();
        apiJwkSet.setKeys(List.of(portalSignatureKey));

        wireMockServer.stubFor(
                get(urlEqualTo("/.well-known/jwks.json"))
                        .willReturn(ok()
                                .withBody(new ObjectMapper().writeValueAsString(apiJwkSet))
                                .withHeader("Content-Type", MediaType.APPLICATION_JSON_VALUE)
                                .withStatus(200)));

        // When
        final ApiJwkSet portalKeys = this.underTest.getPublicKeys();


        // Then
        assertNotNull(portalKeys);
        assertFalse(portalKeys.getKeys().isEmpty());
        assertThat(portalKeys.getKeys().get(0), is(portalSignatureKey));
    }
}